<?php

/**
 * Plugin Name: Cosmic Blocks
 * Plugin URI: https://www.cosmicwp.com/cosmic-blocks
 * Description: High quality blocks collection of 40+ customizable WordPress 5.0 gutenberg content blocks for the new block editor.
 * Version: 0.44
 * Author: Cosmic WP
 * Author URI: https://www.cosmicwp.com
 */
if(!defined('ABSPATH'))
{
	exit;
}

define('COSMIC_BLOCKS_PLUGIN_VERSION', '0.44');
define('COSMIC_BLOCKS_PLUGIN_DIR', dirname(__FILE__));
define('COSMIC_BLOCKS_PLUGIN_URL', plugins_url('', __FILE__));

require_once COSMIC_BLOCKS_PLUGIN_DIR . '/cosmic-blocks.register.php';
require_once COSMIC_BLOCKS_PLUGIN_DIR . '/cosmic-blocks.settings.php';
require_once COSMIC_BLOCKS_PLUGIN_DIR . '/cosmic-blocks.shortcodes.php';

// On plugin activation
register_activation_hook(__FILE__, 'cosmic_blocks_activation');

function cosmic_blocks_activation()
{
	CosmicBlocks_Register::activate();
}

// On plugin deactivation
register_deactivation_hook(__FILE__, 'cosmic_blocks_deactivation');

function cosmic_blocks_deactivation()
{
	CosmicBlocks_Register::deactivate();
}

// On plugin remove
register_uninstall_hook(__FILE__, 'cosmic_blocks_uninstall');

function cosmic_blocks_uninstall()
{
	CosmicBlocks_Register::uninstall();
}

// Run plugin
add_action('plugins_loaded', 'initCosmicBlocks');

function initCosmicBlocks()
{
	new CosmicBlocks;
}

class CosmicBlocks
{

	function __construct()
	{
		// include language
		add_action('init', [$this, 'load_textdomain']);

		// add block category in gutenberg
		add_filter('block_categories', [$this, 'block_category'], 10, 2);

		// add php vars to head
		add_action('admin_head', [$this, 'admin_head']);

		// register blocks js, css
		add_action('init', [$this, 'init_register_blocks']);
		add_action('wp_enqueue_scripts', [$this, 'both_editor_style_script']);
		add_action('wp_enqueue_scripts', [$this, 'front_editor_style_script']);
		add_action('enqueue_block_editor_assets', [$this, 'both_editor_style_script']);
		add_action('enqueue_block_editor_assets', [$this, 'admin_editor_style_script']);
		add_action('admin_enqueue_scripts', [$this, 'common_admin_enqueue_scripts']);
		add_action('admin_head', [$this, 'common_admin_custom_styles']);

		// custompages for stripe checkout
		add_action('init', [$this, 'init_custompages']);
		add_filter('query_vars', [$this, 'query_vars_custompages']);
		add_action('parse_request', [$this, 'parse_request_custompages']);

		// thumbnail sizes
		add_action('after_setup_theme', [$this, 'theme_setup']);
		add_filter('image_size_names_choose', [$this, 'custom_image_sizes']);
		add_filter('image_resize_dimensions', [$this, 'thumbnail_upscale'], 10, 6);

		// thumbnail helper for blocks
		add_action('wp_ajax_cosmic_blocks_thumbnail_helper', [$this, 'wp_ajax_cosmic_blocks_thumbnail_helper']);
		add_action('wp_ajax_cosmic_blocks_thumbnail_helper_single', [$this, 'wp_ajax_cosmic_blocks_thumbnail_helper_single']);

		// shop update notice
		add_action('upgrader_process_complete', [$this, 'update_notice'], 10, 2);
		add_action('admin_notices', [$this, 'update_admin_notices']);

		// show missing dependecies
		add_action('wp_ajax_cosmic_blocks_admin_notice_dismiss', [$this, 'wp_ajax_cosmic_blocks_admin_notice_dismiss']);
		if(get_option('cosmic_blocks_dependency_check') === '1')
		{
			add_action('admin_notices', [$this, 'admin_notice']);
		}

		// settings page
		new CosmicBlocks_Settings;

		// shortcodes
		new CosmicBlocks_Shortcodes;
	}

	/**
	 * load textdomain
	 */
	function load_textdomain()
	{
		// vars
		$domain = 'cosmic-blocks';
		$locale = is_admin() && function_exists('get_user_locale') ? get_user_locale() : get_locale();
		$mofile = $domain . '-' . $locale . '.mo';

		// load from the languages directory first
		load_textdomain($domain, WP_LANG_DIR . '/plugins/' . $mofile);

		// redirect missing translations
		$mofile = str_replace('de_DE', 'de_CH', $mofile);

		// load from plugin lang folder
		$loaded = load_textdomain($domain, COSMIC_BLOCKS_PLUGIN_DIR . '/languages/' . $mofile);
	}

	/**
	 * Custom category
	 */
	public function block_category($categories, $post)
	{

		$cats = [];

		$i = 0;
		foreach($categories as $category)
		{
			if($i++ === 1)
			{
				$cats[] = [
					'slug' => 'cwp-blocks',
					'title' => __('Cosmic Blocks'),
				];
			}

			$cats[] = $category;
		}

		return $cats;
	}

	/**
	 * backend script
	 * backend style
	 */
	function admin_head()
	{
		$jsOptions['plugin_url'] = COSMIC_BLOCKS_PLUGIN_URL;
		echo '<script type="text/javascript">var cwp_options = ' . json_encode($jsOptions) . ';</script>';

		$smallSize = get_option('cosmic_blocks_editor_block_width') ?: '610px';
		$wideSize = get_option('cosmic_blocks_editor_block_wide_width') ?: '1100px';

		$cssOptions = '';
		//$cssOptions .= '.wp-block { background:yellow !important; }';
		//$cssOptions .= '.wp-block .cwp-common-main-wrap { background:orange !important; }';
		if(get_option('cosmic_blocks_editor_block') !== '1')
		{
			$cssOptions .= '.wp-block { max-width:' . $smallSize . ' !important; }';
			$cssOptions .= '.wp-block[data-align="wide"] { max-width:' . $wideSize . ' !important; }';
			$cssOptions .= '.wp-block[data-align="full"] { max-width:none !important; }';
		}
		$cssOptions .= '.cwp-full-width-small-container { max-width:100% !important; }';
		$cssOptions .= '.cwp-full-width-small-container .cwp-common-main-wrap-inner { max-width:' . $smallSize . ' !important; margin-left: auto !important; margin-right: auto !important; padding-left:14px !important; padding-right:14px !important; }';
		$cssOptions .= '.cwp-full-width-small-container .cwp-common-main-wrap-inner .wp-block { max-width:none !important; margin-left: auto !important; margin-right: auto !important; }';
		$cssOptions .= '.cwp-full-width-wide-container { max-width:100% !important; }';
		$cssOptions .= '.cwp-full-width-wide-container .cwp-common-main-wrap-inner { max-width:' . $wideSize . ' !important; margin-left: auto !important; margin-right: auto !important; padding-left:14px !important; padding-right:14px !important; }';
		$cssOptions .= '.cwp-full-width-wide-container .cwp-common-main-wrap-inner .wp-block { max-width:none !important; margin-left: auto !important; margin-right: auto !important; }';
		$cssOptions .= '.cwp-same-width-small-container { max-width:' . $smallSize . ' !important; margin-left: auto !important; margin-right: auto !important; }';
		$cssOptions .= '.cwp-same-width-small-container .cwp-common-main-wrap-inner { max-width:100% !important; }';
		$cssOptions .= '.cwp-same-width-small-container .cwp-common-main-wrap-inner .wp-block { max-width:none !important; margin-left: auto !important; margin-right: auto !important; }';
		$cssOptions .= '.cwp-same-width-wide-container { max-width:' . $wideSize . ' !important; margin-left: auto !important; margin-right: auto !important; }';
		$cssOptions .= '.cwp-same-width-wide-container .cwp-common-main-wrap-inner { max-width:100% !important; }';
		$cssOptions .= '.cwp-same-width-wide-container .cwp-common-main-wrap-inner .wp-block { max-width:none !important; margin-left: auto !important; margin-right: auto !important; }';
		$cssOptions .= '.cwp-same-width-full-container { max-width:100% !important; margin-left: auto !important; margin-right: auto !important; }';
		$cssOptions .= '.cwp-same-width-full-container .cwp-common-main-wrap-inner { max-width:100% !important; }';
		$cssOptions .= '.cwp-same-width-full-container .cwp-common-main-wrap-inner .wp-block { max-width:none !important; margin-left: auto !important; margin-right: auto !important; }';
		if($cssOptions)
		{
			echo '<style>' . $cssOptions . '</style>';
		}
	}

	/**
	 * Add custom pages
	 */
	function init_custompages()
	{
		add_rewrite_rule('^stripe/payment', 'index.php?cosmic-blocks-pages=payment', 'top');
		add_rewrite_rule('^cwp-cosmic-blocks/fix-noopener', 'index.php?cosmic-blocks-pages=fix-noopener', 'top');

		if(!get_option('cosmic_blocks_permalinks_flushed'))
		{
			flush_rewrite_rules(false);
			CosmicBlocks_Register::flushed(true);
		}
	}

	/**
	 * Add custom pages to query vars
	 * @param array $query_vars
	 * @return string
	 */
	function query_vars_custompages($query_vars)
	{
		$query_vars[] = 'cosmic-blocks-pages';
		return $query_vars;
	}

	// load custom page
	function parse_request_custompages(&$wp)
	{
		if(array_key_exists('cosmic-blocks-pages', $wp->query_vars))
		{
			require_once COSMIC_BLOCKS_PLUGIN_DIR . '/custompages/' . $wp->query_vars['cosmic-blocks-pages'] . '.php';
			exit();
		}
	}

	/**
	 * Add admin and frontend style/script
	 */
	function both_editor_style_script()
	{
		// base styles
		wp_enqueue_style('cwp-base-style', COSMIC_BLOCKS_PLUGIN_URL . '/assets/css/cosmic-blocks.style.css', [], COSMIC_BLOCKS_PLUGIN_VERSION, 'screen');

		// vendor styles
		wp_enqueue_style('animate', COSMIC_BLOCKS_PLUGIN_URL . '/assets/css/animate.min.css', [], '3.6.0', 'screen');
		wp_enqueue_style('fontawesome4', COSMIC_BLOCKS_PLUGIN_URL . '/assets/css/font-awesome.min.css', [], '4.7.0', 'screen');
		wp_enqueue_style('slick', COSMIC_BLOCKS_PLUGIN_URL . '/assets/css/slick.min.css', [], '1.9.0', 'screen');
		wp_enqueue_style('slick-theme', COSMIC_BLOCKS_PLUGIN_URL . '/assets/css/slick-theme.min.css', [], '1.9.0', 'screen');
		//wp_enqueue_style('leaflet', COSMIC_BLOCKS_PLUGIN_URL.'/assets/leaflet/leaflet.css', [], '1.4.0', 'screen');
		// vendor scripts
		wp_enqueue_script('scroll', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/scrollClass.js', ['jquery'], COSMIC_BLOCKS_PLUGIN_VERSION, true);
		wp_enqueue_script('slick', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/slick.min.js', ['jquery'], '1.9.0', true);
		//wp_enqueue_script('leaflet', COSMIC_BLOCKS_PLUGIN_URL.'/assets/leaflet/leaflet.js', [], '1.4.0', true);

		wp_enqueue_script('waypoints', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/waypoints.min.js', ['jquery'], '4.0.1', true);
		//wp_enqueue_script('stripe-checkout', 'https://checkout.stripe.com/checkout.js', ['jquery'], COSMIC_BLOCKS_PLUGIN_VERSION, true);
	}

	/**
	 * Add frontend only style/script
	 */
	function front_editor_style_script()
	{
		// padding inline styles
		$custom_css = '';
		if(get_option('cosmic_blocks_main_padding_top'))
		{
			$custom_css .= '.cwp-common-main-wrap .cwp-common-main-wrap-inner { padding-top:' . get_option('cosmic_blocks_main_padding_top') . ' }';
		}
		if(get_option('cosmic_blocks_main_padding_bottom'))
		{
			$custom_css .= '.cwp-common-main-wrap .cwp-common-main-wrap-inner { padding-bottom:' . get_option('cosmic_blocks_main_padding_bottom') . ' }';
		}
		if(get_option('cosmic_blocks_main_intro_padding_bottom'))
		{
			$custom_css .= '.cwp-common-main-wrap .cwp-common-main-wrap-inner .cwp-main-intro { padding-bottom:' . get_option('cosmic_blocks_main_intro_padding_bottom') . ' }';
		}
		if($custom_css)
		{
			wp_add_inline_style('cwp-base-style', $custom_css);
		}

		// mobile padding inline styles
		$custom_css_mobile = '';
		if(get_option('cosmic_blocks_mobile_main_padding_top'))
		{
			$custom_css_mobile .= '.cwp-common-main-wrap .cwp-common-main-wrap-inner { padding-top:' . get_option('cosmic_blocks_mobile_main_padding_top') . ' }';
		}
		if(get_option('cosmic_blocks_mobile_main_padding_bottom'))
		{
			$custom_css_mobile .= '.cwp-common-main-wrap .cwp-common-main-wrap-inner { padding-bottom:' . get_option('cosmic_blocks_mobile_main_padding_bottom') . ' }';
		}
		if(get_option('cosmic_blocks_mobile_main_intro_padding_bottom'))
		{
			$custom_css_mobile .= '.cwp-common-main-wrap .cwp-common-main-wrap-inner { padding-bottom:' . get_option('cosmic_blocks_mobile_main_intro_padding_bottom') . ' }';
		}
		if($custom_css_mobile)
		{
			wp_add_inline_style('cwp-base-style', '@media(max-width:991px){' . $custom_css_mobile) . '}';
		}

		// container size inline styles
		if(get_option('cosmic_blocks_front_block') === 'simple')
		{
			$smallSize = get_option('cosmic_blocks_front_block_simple_small') ?: '930px';
			$wideSize = get_option('cosmic_blocks_front_block_simple_wide') ?: '1110px';
			wp_add_inline_style('cwp-base-style', '.cwp-full-width-small-container{max-width:100%!important}.cwp-full-width-small-container .cwp-common-main-wrap-inner{max-width:' . $smallSize . '!important;margin:0 auto!important}.cwp-full-width-wide-container{max-width:100%!important}.cwp-full-width-wide-container .cwp-common-main-wrap-inner{max-width:' . $wideSize . '!important;margin:0 auto!important}.cwp-same-width-small-container{max-width:' . $smallSize . '!important;margin:0 auto!important}.cwp-same-width-small-container .cwp-common-main-wrap-inner{max-width:100%!important}.cwp-same-width-wide-container{max-width:' . $wideSize . '!important;margin:0 auto!important}.cwp-same-width-full-container .cwp-common-main-wrap-inner,.cwp-same-width-wide-container .cwp-common-main-wrap-inner{max-width:100%!important}.cwp-same-width-full-container{max-width:100%!important;margin:0 auto!important}');
		}
		else if(get_option('cosmic_blocks_front_block') === 'bootstrap4')
		{
			wp_add_inline_style('cwp-base-style', '.cwp-full-width-small-container{max-width:100% !important}.cwp-full-width-small-container .cwp-common-main-wrap-inner{max-width:100%;margin:0 auto}.cwp-full-width-wide-container{max-width:100% !important}.cwp-full-width-wide-container .cwp-common-main-wrap-inner{max-width:100%;margin:0 auto}.cwp-same-width-small-container{max-width:100% !important;margin:0 auto !important}.cwp-same-width-small-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-wide-container{max-width:100% !important;margin:0 auto !important}.cwp-same-width-wide-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-full-container{max-width:100% !important}.cwp-same-width-full-container .cwp-common-main-wrap-inner{max-width:100%}@media screen and (min-width:576px){.cwp-full-width-small-container{max-width:100% !important}.cwp-full-width-small-container .cwp-common-main-wrap-inner{max-width:510px;margin:0 auto}.cwp-full-width-wide-container{max-width:100% !important}.cwp-full-width-wide-container .cwp-common-main-wrap-inner{max-width:510px;margin:0 auto}.cwp-same-width-small-container{max-width:510px !important;margin:0 auto !important}.cwp-same-width-small-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-wide-container{max-width:510px !important;margin:0 auto !important}.cwp-same-width-wide-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-full-container{max-width:100% !important}.cwp-same-width-full-container .cwp-common-main-wrap-inner{max-width:100%}}@media screen and (min-width:768px){.cwp-full-width-small-container{max-width:100% !important}.cwp-full-width-small-container .cwp-common-main-wrap-inner{max-width:690px;margin:0 auto}.cwp-full-width-wide-container{max-width:100% !important}.cwp-full-width-wide-container .cwp-common-main-wrap-inner{max-width:690px;margin:0 auto}.cwp-same-width-small-container{max-width:690px !important;margin:0 auto !important}.cwp-same-width-small-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-wide-container{max-width:690px !important;margin:0 auto !important}.cwp-same-width-wide-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-full-container{max-width:100% !important}.cwp-same-width-full-container .cwp-common-main-wrap-inner{max-width:100%}}@media screen and (min-width:992px){.cwp-full-width-small-container{max-width:100% !important}.cwp-full-width-small-container .cwp-common-main-wrap-inner{max-width:930px;margin:0 auto}.cwp-full-width-wide-container{max-width:100% !important}.cwp-full-width-wide-container .cwp-common-main-wrap-inner{max-width:930px;margin:0 auto}.cwp-same-width-small-container{max-width:930px !important;margin:0 auto !important}.cwp-same-width-small-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-wide-container{max-width:930px !important;margin:0 auto !important}.cwp-same-width-wide-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-full-container{max-width:100% !important}.cwp-same-width-full-container .cwp-common-main-wrap-inner{max-width:100%}}@media screen and (min-width:1200px){.cwp-full-width-small-container{max-width:100% !important}.cwp-full-width-small-container .cwp-common-main-wrap-inner{max-width:930px;margin:0 auto}.cwp-full-width-wide-container{max-width:100% !important}.cwp-full-width-wide-container .cwp-common-main-wrap-inner{max-width:1110px;margin:0 auto}.cwp-same-width-small-container{max-width:930px !important;margin:0 auto !important}.cwp-same-width-small-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-wide-container{max-width:1110px !important;margin:0 auto !important}.cwp-same-width-wide-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-full-container{max-width:100% !important}.cwp-same-width-full-container .cwp-common-main-wrap-inner{max-width:100%}}');
		}
		else if(get_option('cosmic_blocks_front_block') === 'custom')
		{
			$customQuery = get_option('cosmic_blocks_front_block_custom_query') ?: [];
			$customSmall = get_option('cosmic_blocks_front_block_custom_small') ?: [];
			$customWide = get_option('cosmic_blocks_front_block_custom_wide') ?: [];
			$custom_css = '';

			$i = 0;
			while($i < count($customQuery))
			{
				if(isset($customSmall[$i]) && isset($customWide[$i]))
				{
					$custom_css .= '@media screen and (min-width:' . $customQuery[$i] . '){.cwp-full-width-small-container{max-width:100% !important}.cwp-full-width-small-container .cwp-common-main-wrap-inner{max-width:930px;margin:0 auto}.cwp-full-width-wide-container{max-width:100% !important}.cwp-full-width-wide-container .cwp-common-main-wrap-inner{max-width:' . $customWide[$i] . ';margin:0 auto}.cwp-same-width-small-container{max-width:930px !important;margin:0 auto !important}.cwp-same-width-small-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-wide-container{max-width:' . $customWide[$i] . ' !important;margin:0 auto !important}.cwp-same-width-wide-container .cwp-common-main-wrap-inner{max-width:100%}.cwp-same-width-full-container{max-width:100% !important}.cwp-same-width-full-container .cwp-common-main-wrap-inner{max-width:100%}}';
				}
				$i++;
			}
			if($custom_css)
			{
				wp_add_inline_style('cwp-base-style', $custom_css);
			}
		}

		// frontend script
		wp_register_script('cwp-front-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/front.js', ['jquery'], COSMIC_BLOCKS_PLUGIN_VERSION, true);
		wp_localize_script('cwp-front-script', 'php_vars', [
			'stripeName' => get_bloginfo('name'),
			'stripeKey' => get_option('cosmic_blocks_donations_stripe_publishable'),
			'stripeCurrency' => get_option('cosmic_blocks_donations_stripe_currency'),
			'stripeUrl' => get_site_url() . '/stripe/payment'
		]);
		wp_enqueue_script('cwp-front-script');
	}

	/**
	 * common admin styles (for settings page)
	 */
	function common_admin_enqueue_scripts()
	{
		wp_enqueue_style('cwp-admin-style', COSMIC_BLOCKS_PLUGIN_URL . '/assets/css/cosmic-blocks.admin.css', [], COSMIC_BLOCKS_PLUGIN_VERSION);
		wp_enqueue_script('cwp-admin-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/back.js', ['jquery'], COSMIC_BLOCKS_PLUGIN_VERSION);

		wp_enqueue_style('font-icon-picker-theme', COSMIC_BLOCKS_PLUGIN_URL . '/assets/ext/fontIconPicker-react/fonticonpicker.base-theme.react.css', [], '1.2.0');
		wp_enqueue_script('font-icon-picker', COSMIC_BLOCKS_PLUGIN_URL . '/assets/ext/fontIconPicker-react/fonticonpicker.react.js', [], '1.2.0');
	}

	/**
	 * common admin styles (for settings page)
	 */
	function common_admin_custom_styles()
	{
		$custom_css = '';
		if(get_option('cosmic_blocks_main_padding_top'))
		{
			$custom_css .= '.cwp-common-main-wrap .cwp-common-main-wrap-inner { padding-top:' . get_option('cosmic_blocks_main_padding_top') . ' }';
		}
		if(get_option('cosmic_blocks_main_padding_bottom'))
		{
			$custom_css .= '.cwp-common-main-wrap .cwp-common-main-wrap-inner { padding-bottom:' . get_option('cosmic_blocks_main_padding_bottom') . ' }';
		}
		if(get_option('cosmic_blocks_main_intro_padding_bottom'))
		{
			$custom_css .= '.cwp-common-main-wrap .cwp-common-main-wrap-inner .cwp-main-intro { padding-bottom:' . get_option('cosmic_blocks_main_intro_padding_bottom') . ' }';
		}
		if($custom_css)
		{
			echo '<style>' . $custom_css . '</style>';
		}
	}

	/**
	 * Add admin only style/script
	 */
	function admin_editor_style_script()
	{
		// register editor styles
		wp_enqueue_style('cwp-editor-style', COSMIC_BLOCKS_PLUGIN_URL . '/assets/css/cosmic-blocks.editor.css', ['wp-edit-blocks'], COSMIC_BLOCKS_PLUGIN_VERSION);

		// register block scripts
		wp_enqueue_script('cwp-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/common.js', ['jquery', 'wp-blocks', 'wp-element', 'wp-editor', 'wp-i18n'], COSMIC_BLOCKS_PLUGIN_VERSION);
		wp_set_script_translations('cwp-script', 'cosmic-blocks', COSMIC_BLOCKS_PLUGIN_DIR . '/languages');
		wp_localize_script('cwp-script', 'cwp_options', [
			'plugin_url' => COSMIC_BLOCKS_PLUGIN_URL
		]);

		wp_register_script('cwp-accordion-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-accordion.js', [], COSMIC_BLOCKS_PLUGIN_VERSION);
		wp_register_script('cwp-advantages-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-advantages.js', []);
		wp_set_script_translations('cwp-advantages-script', 'cosmic-blocks', COSMIC_BLOCKS_PLUGIN_DIR . '/languages');

		wp_register_script('cwp-brands-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-brands.js', []);
		wp_register_script('cwp-buttons-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-buttons.js', []);
		wp_register_script('cwp-classifieds-list-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-classifieds-list.js', []);
		wp_register_script('cwp-classifieds-tiles-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-classifieds-tiles.js', []);
		wp_register_script('cwp-contacts-list-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-contacts-list.js', []);
		wp_register_script('cwp-contacts-tiles-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-contacts-tiles.js', []);
		wp_register_script('cwp-contacts-target-bar-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-contacts-target-bar.js', []);
		wp_register_script('cwp-container-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-container.js', []);
		wp_register_script('cwp-container-fancy-animated-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-container-fancy-animated.js', []);
		wp_register_script('cwp-countdown-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-countdown.js', []);
		wp_register_script('cwp-countup-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-countup.js', []);
		wp_register_script('cwp-date-list-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-date-list.js', []);
		wp_register_script('cwp-date-tiles-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-date-tiles.js', []);
		wp_register_script('cwp-deals-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-deals.js', []);
		wp_register_script('cwp-donations-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-donations.js', []);
		wp_register_script('cwp-employee-list-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-employee-list.js', []);
		wp_register_script('cwp-employee-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-employee.js', []);
		wp_register_script('cwp-events-list-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-events-list.js', []);
		wp_register_script('cwp-events-tiles-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-events-tiles.js', []);
		wp_register_script('cwp-events-upcoming-list-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-events-upcoming-list.js', []);
		wp_register_script('cwp-events-upcoming-tiles-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-events-upcoming-tiles.js', []);
		wp_register_script('cwp-facts-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-facts.js', []);
		wp_register_script('cwp-foreword-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-foreword.js', []);
		wp_register_script('cwp-greeting-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-greeting.js', []);
		wp_register_script('cwp-image-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-image.js', []);
		wp_register_script('cwp-inline-notice-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-inline-notice.js', []);
		wp_register_script('cwp-launch-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-launch.js', []);
		wp_register_script('cwp-like-dislike-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-like-dislike.js', []);
		wp_register_script('cwp-links-download-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-links-download.js', []);
		wp_register_script('cwp-links-icon-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-links-icon.js', []);
		wp_register_script('cwp-links-image-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-links-image.js', []);
		wp_register_script('cwp-links-image-slide-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-links-image-slide.js', []);
		wp_register_script('cwp-links-list-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-links-list.js', []);
		//wp_register_script('cwp-openstreetmap-script', COSMIC_BLOCKS_PLUGIN_URL.'/assets/js/block-openstreetmap.js', []);
		wp_register_script('cwp-posts-list-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-posts-list.js', []);
		wp_register_script('cwp-posts-tiles-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-posts-tiles.js', []);
		wp_register_script('cwp-pricebox-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-pricebox.js', []);
		wp_register_script('cwp-pricelist-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-pricelist.js', []);
		wp_register_script('cwp-progress-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-progress.js', []);
		wp_register_script('cwp-separator-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-separator.js', []);
		wp_register_script('cwp-showcase-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-showcase.js', []);
		wp_register_script('cwp-social-share-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-social-share.js', []);
		wp_register_script('cwp-split-list-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-split-list.js', []);
		wp_register_script('cwp-statement-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-statement.js', []);
		wp_register_script('cwp-steps-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-steps.js', []);
		wp_register_script('cwp-testimonial-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-testimonial.js', []);
		wp_register_script('cwp-title-intro-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-title-intro.js', []);
		wp_register_script('cwp-voting-slogan-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-voting-slogan.js', []);
		wp_register_script('cwp-wanted-script', COSMIC_BLOCKS_PLUGIN_URL . '/assets/js/block-wanted.js', []);
	}

	function init_register_blocks()
	{
		// Accordion
		if(get_option('cosmic_blocks_accordion') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/accordion', [
					'editor_script' => 'cwp-accordion-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Advantages
		if(get_option('cosmic_blocks_advantages') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/advantages', [
					'editor_script' => 'cwp-advantages-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Brands
		if(get_option('cosmic_blocks_brands') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/brands', [
					'editor_script' => 'cwp-brands-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Buttons
		if(get_option('cosmic_blocks_buttons') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/buttons', [
					'editor_script' => 'cwp-buttons-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Classifieds List
		if(get_option('cosmic_blocks_classifieds_list') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/classifieds-list', [
					'editor_script' => 'cwp-classifieds-list-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Classifieds Tiles
		if(get_option('cosmic_blocks_classifieds_tiles') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/classifieds-tiles', [
					'editor_script' => 'cwp-classifieds-tiles-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Contacts List
		if(get_option('cosmic_blocks_contacts_list') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/contacts-list', [
					'editor_script' => 'cwp-contacts-list-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
					'render_callback' => 'cwp_render_contacts_list',
					'attributes' => [
						'preview' => ['type' => 'boolean', 'default' => false],
						'blockID' => ['type' => 'string', 'default' => ''],
						'columns' => ['type' => 'number', 'default' => 1],
						'container' => ['type' => 'string', 'default' => 'cwp-full-width-wide-container'],
						'startClass' => ['type' => 'string', 'default' => 'cwp-all-left'],
						'backgroundType' => ['type' => 'string', 'default' => 'color'],
						'backgroundTextColor' => ['type' => 'string', 'default' => ''],
						'solidColor' => ['type' => 'string', 'default' => ''],
						'gradient' => ['type' => 'string', 'default' => ''],
						'backgroundMediaID' => ['type' => 'number', 'default' => ''],
						'backgroundMediaURL' => ['type' => 'string', 'default' => ''],
						'animationStyle' => ['type' => 'string', 'default' => ''],
						'animationDelay' => ['type' => 'number', 'default' => 10],
						'threshold' => ['type' => 'number', 'default' => 40],
						'paddingBeforeIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterItems' => ['type' => 'number', 'default' => 50],
						'introsection' => ['type' => 'string', 'default' => 'empty'],
						'introtitle' => ['type' => 'string', 'default' => ''],
						'introsubtitle' => ['type' => 'string', 'default' => ''],
						'introduction' => ['type' => 'string', 'default' => ''],
						'boxradius' => ['type' => 'number', 'default' => 0],
						'boxradiusfix' => ['type' => 'boolean', 'default' => false],
						'boxpadding' => ['type' => 'number', 'default' => 0],
						'boxpaddingfix' => ['type' => 'boolean', 'default' => false],
						'selectedCategory' => ['type' => 'string', 'default' => ''],
						'postsPerPage' => ['type' => 'string', 'default' => '25'],
						'filter' => ['type' => 'string', 'default' => 'all'],
						'order' => ['type' => 'string', 'default' => 'cosmic_campaigns_contact_last_name'],
						'showImage' => ['type' => 'boolean', 'default' => true],
						'showSalutation' => ['type' => 'boolean', 'default' => false],
						'showSuffix' => ['type' => 'boolean', 'default' => false],
						'showName' => ['type' => 'boolean', 'default' => true],
						'showFunction' => ['type' => 'boolean', 'default' => true],
						'showStreetAndNumber' => ['type' => 'boolean', 'default' => false],
						'showZipAndPlace' => ['type' => 'boolean', 'default' => true],
						'showCountry' => ['type' => 'boolean', 'default' => false],
						'showEmail' => ['type' => 'boolean', 'default' => false],
						'showMobile' => ['type' => 'boolean', 'default' => false],
						'showPhonePrivate' => ['type' => 'boolean', 'default' => false],
						'showPhoneWork' => ['type' => 'boolean', 'default' => false],
						'showWebsiteAndSocialMedia' => ['type' => 'boolean', 'default' => false],
						'showWebsiteAndSocialMediaStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-white'],
						'showPagination' => ['type' => 'boolean', 'default' => true],
						'paginationStyle' => ['type' => 'string', 'default' => 'cwp-pagination cwp-pagination-secondary'],
						'textcolor' => ['type' => 'string', 'default' => ''],
						'color' => ['type' => 'string', 'default' => ''],
					]
				]);
			}
		}

		// Contacts Tiles
		if(get_option('cosmic_blocks_contacts_tiles') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/contacts-tiles', [
					'editor_script' => 'cwp-contacts-tiles-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
					'render_callback' => 'cwp_render_contacts_tiles',
					'attributes' => [
						'preview' => ['type' => 'boolean', 'default' => false],
						'blockID' => ['type' => 'string', 'default' => ''],
						'columns' => ['type' => 'number', 'default' => 3],
						'container' => ['type' => 'string', 'default' => 'cwp-full-width-wide-container'],
						'backgroundType' => ['type' => 'string', 'default' => 'color'],
						'backgroundTextColor' => ['type' => 'string', 'default' => ''],
						'solidColor' => ['type' => 'string', 'default' => ''],
						'gradient' => ['type' => 'string', 'default' => ''],
						'backgroundMediaID' => ['type' => 'number', 'default' => ''],
						'backgroundMediaURL' => ['type' => 'string', 'default' => ''],
						'animationStyle' => ['type' => 'string', 'default' => ''],
						'animationDelay' => ['type' => 'number', 'default' => 10],
						'threshold' => ['type' => 'number', 'default' => 40],
						'paddingBeforeIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterItems' => ['type' => 'number', 'default' => 50],
						'introsection' => ['type' => 'string', 'default' => 'empty'],
						'introtitle' => ['type' => 'string', 'default' => ''],
						'introsubtitle' => ['type' => 'string', 'default' => ''],
						'introduction' => ['type' => 'string', 'default' => ''],
						'boxradius' => ['type' => 'number', 'default' => 0],
						'boxradiusfix' => ['type' => 'boolean', 'default' => false],
						'boxpadding' => ['type' => 'number', 'default' => 0],
						'boxpaddingfix' => ['type' => 'boolean', 'default' => false],
						'selectedCategory' => ['type' => 'string', 'default' => ''],
						'postsPerPage' => ['type' => 'string', 'default' => '25'],
						'filter' => ['type' => 'string', 'default' => 'all'],
						'order' => ['type' => 'string', 'default' => 'cosmic_campaigns_contact_last_name'],
						'showImage' => ['type' => 'boolean', 'default' => false],
						'showSalutation' => ['type' => 'boolean', 'default' => false],
						'showSuffix' => ['type' => 'boolean', 'default' => false],
						'showName' => ['type' => 'boolean', 'default' => true],
						'showFunction' => ['type' => 'boolean', 'default' => true],
						'showStreetAndNumber' => ['type' => 'boolean', 'default' => false],
						'showZipAndPlace' => ['type' => 'boolean', 'default' => true],
						'showCountry' => ['type' => 'boolean', 'default' => false],
						'showEmail' => ['type' => 'boolean', 'default' => false],
						'showMobile' => ['type' => 'boolean', 'default' => false],
						'showPhonePrivate' => ['type' => 'boolean', 'default' => false],
						'showPhoneWork' => ['type' => 'boolean', 'default' => false],
						'showWebsiteAndSocialMedia' => ['type' => 'boolean', 'default' => false],
						'showWebsiteAndSocialMediaStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-white'],
						'showPagination' => ['type' => 'boolean', 'default' => true],
						'paginationStyle' => ['type' => 'string', 'default' => 'cwp-pagination cwp-pagination-secondary'],
						'textcolor' => ['type' => 'string', 'default' => ''],
						'color' => ['type' => 'string', 'default' => ''],
					]
				]);
			}
		}

		// Contacts Target Bar
		if(get_option('cosmic_blocks_contacts_target_bar') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/contacts-target-bar', [
					'editor_script' => 'cwp-contacts-target-bar-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
					'render_callback' => 'cwp_render_contacts_target_bar',
					'attributes' => [
						'preview' => ['type' => 'boolean', 'default' => false],
						'blockID' => ['type' => 'string', 'default' => ''],
						'columns' => ['type' => 'number', 'default' => 1],
						'container' => ['type' => 'string', 'default' => 'cwp-full-width-wide-container'],
						'backgroundType' => ['type' => 'string', 'default' => 'color'],
						'backgroundTextColor' => ['type' => 'string', 'default' => ''],
						'solidColor' => ['type' => 'string', 'default' => ''],
						'gradient' => ['type' => 'string', 'default' => ''],
						'backgroundMediaID' => ['type' => 'number', 'default' => ''],
						'backgroundMediaURL' => ['type' => 'string', 'default' => ''],
						'animationStyle' => ['type' => 'string', 'default' => ''],
						'animationDelay' => ['type' => 'number', 'default' => 10],
						'threshold' => ['type' => 'number', 'default' => 40],
						'paddingBeforeIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterItems' => ['type' => 'number', 'default' => 50],
						'introsection' => ['type' => 'string', 'default' => 'empty'],
						'introtitle' => ['type' => 'string', 'default' => ''],
						'introsubtitle' => ['type' => 'string', 'default' => ''],
						'introduction' => ['type' => 'string', 'default' => ''],
						'boxradius' => ['type' => 'number', 'default' => 0],
						'boxradiusfix' => ['type' => 'boolean', 'default' => false],
						'boxpadding' => ['type' => 'number', 'default' => 0],
						'boxpaddingfix' => ['type' => 'boolean', 'default' => false],
						'title' => ['type' => 'string', 'default' => ''],
						'textbefore' => ['type' => 'string', 'default' => ''],
						'filter' => ['type' => 'string', 'default' => 'all'],
						'target' => ['type' => 'string', 'default' => 'dynamic'],
						'targetValue' => ['type' => 'string', 'default' => '10000'],
						'textafter' => ['type' => 'string', 'default' => ''],
						'progressStyle' => ['type' => 'string', 'default' => 'cwp-progress cwp-progress-primary'],
						'textcolor' => ['type' => 'string', 'default' => ''],
						'color' => ['type' => 'string', 'default' => ''],
					]
				]);
			}
		}

		// Container
		if(get_option('cosmic_blocks_container') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/container', [
					'editor_script' => 'cwp-container-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Container Fancy Animated
		if(get_option('cosmic_blocks_container_fancy_animated') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/container-fancy-animated', [
					'editor_script' => 'cwp-container-fancy-animated-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Countdown
		if(get_option('cosmic_blocks_countdown') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/countdown', [
					'editor_script' => 'cwp-countdown-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Count Up
		if(get_option('cosmic_blocks_countup') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/countup', [
					'editor_script' => 'cwp-countup-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Date List
		if(get_option('cosmic_blocks_date_list') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/date-list', [
					'editor_script' => 'cwp-date-list-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Date Tiles
		if(get_option('cosmic_blocks_date_tiles') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/date-tiles', [
					'editor_script' => 'cwp-date-tiles-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Deals
		if(get_option('cosmic_blocks_deals') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/deals', [
					'editor_script' => 'cwp-deals-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Donations
		if(get_option('cosmic_blocks_donations') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/donations', [
					'editor_script' => 'cwp-donations-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Employee List
		if(get_option('cosmic_blocks_employee_list') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/employee-list', [
					'editor_script' => 'cwp-employee-list-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Employee
		if(get_option('cosmic_blocks_employee') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/employee', [
					'editor_script' => 'cwp-employee-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Events List
		if(get_option('cosmic_blocks_events_list') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/events-list', [
					'editor_script' => 'cwp-events-list-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
					'render_callback' => 'cwp_render_events_list',
					'attributes' => [
						'preview' => ['type' => 'boolean', 'default' => false],
						'blockID' => ['type' => 'string', 'default' => ''],
						'columns' => ['type' => 'number', 'default' => 1],
						'container' => ['type' => 'string', 'default' => 'cwp-full-width-wide-container'],
						'startClass' => ['type' => 'string', 'default' => 'cwp-all-left'],
						'backgroundType' => ['type' => 'string', 'default' => 'color'],
						'backgroundTextColor' => ['type' => 'string', 'default' => ''],
						'solidColor' => ['type' => 'string', 'default' => ''],
						'gradient' => ['type' => 'string', 'default' => ''],
						'backgroundMediaID' => ['type' => 'number', 'default' => ''],
						'backgroundMediaURL' => ['type' => 'string', 'default' => ''],
						'animationStyle' => ['type' => 'string', 'default' => ''],
						'animationDelay' => ['type' => 'number', 'default' => 10],
						'threshold' => ['type' => 'number', 'default' => 40],
						'paddingBeforeIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterItems' => ['type' => 'number', 'default' => 50],
						'introsection' => ['type' => 'string', 'default' => 'empty'],
						'introtitle' => ['type' => 'string', 'default' => ''],
						'introsubtitle' => ['type' => 'string', 'default' => ''],
						'introduction' => ['type' => 'string', 'default' => ''],
						'boxradius' => ['type' => 'number', 'default' => 0],
						'boxradiusfix' => ['type' => 'boolean', 'default' => false],
						'boxpadding' => ['type' => 'number', 'default' => 0],
						'boxpaddingfix' => ['type' => 'boolean', 'default' => false],
						'postsPerPage' => ['type' => 'string', 'default' => '25'],
						'position' => ['type' => 'string', 'default' => 'inside'],
						'order' => ['type' => 'string', 'default' => 'ASC'],
						'orderby' => ['type' => 'string', 'default' => 'date'],
						'showTitle' => ['type' => 'boolean', 'default' => true],
						'showExcerpt' => ['type' => 'boolean', 'default' => true],
						'showDate' => ['type' => 'boolean', 'default' => true],
						'showAuthor' => ['type' => 'boolean', 'default' => false],
						'showIcal' => ['type' => 'boolean', 'default' => true],
						'showImage' => ['type' => 'boolean', 'default' => true],
						'imageSize' => ['type' => 'string', 'default' => ''],
						'showButton' => ['type' => 'string', 'default' => 'yes'],
						'showButtonText' => ['type' => 'string', 'default' => __('Read more', 'cosmic-blocks')],
						'showButtonStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-primary'],
						'showButtonTarget' => ['type' => 'string', 'default' => '_self'],
						'showButtonIcal' => ['type' => 'string', 'default' => 'yes'],
						'showButtonIcalText' => ['type' => 'string', 'default' => __('Copy to calendar', 'cosmic-blocks')],
						'showButtonIcalStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-secondary'],
						'showPagination' => ['type' => 'boolean', 'default' => true],
						'paginationStyle' => ['type' => 'string', 'default' => 'cwp-pagination cwp-pagination-secondary'],
						'textcolor' => ['type' => 'string', 'default' => ''],
						'color' => ['type' => 'string', 'default' => ''],
					]
				]);
			}
		}

		// Events Tiles
		if(get_option('cosmic_blocks_events_tiles') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/events-tiles', [
					'editor_script' => 'cwp-events-tiles-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
					'render_callback' => 'cwp_render_events_tiles',
					'attributes' => [
						'preview' => ['type' => 'boolean', 'default' => false],
						'blockID' => ['type' => 'string', 'default' => ''],
						'columns' => ['type' => 'number', 'default' => 3],
						'container' => ['type' => 'string', 'default' => 'cwp-full-width-wide-container'],
						'backgroundType' => ['type' => 'string', 'default' => 'color'],
						'backgroundTextColor' => ['type' => 'string', 'default' => ''],
						'solidColor' => ['type' => 'string', 'default' => ''],
						'gradient' => ['type' => 'string', 'default' => ''],
						'backgroundMediaID' => ['type' => 'number', 'default' => ''],
						'backgroundMediaURL' => ['type' => 'string', 'default' => ''],
						'animationStyle' => ['type' => 'string', 'default' => ''],
						'animationDelay' => ['type' => 'number', 'default' => 10],
						'threshold' => ['type' => 'number', 'default' => 40],
						'paddingBeforeIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterItems' => ['type' => 'number', 'default' => 50],
						'introsection' => ['type' => 'string', 'default' => 'empty'],
						'introtitle' => ['type' => 'string', 'default' => ''],
						'introsubtitle' => ['type' => 'string', 'default' => ''],
						'introduction' => ['type' => 'string', 'default' => ''],
						'boxradius' => ['type' => 'number', 'default' => 0],
						'boxradiusfix' => ['type' => 'boolean', 'default' => false],
						'boxpadding' => ['type' => 'number', 'default' => 0],
						'boxpaddingfix' => ['type' => 'boolean', 'default' => false],
						'postsPerPage' => ['type' => 'string', 'default' => '24'],
						'position' => ['type' => 'string', 'default' => 'after'],
						'order' => ['type' => 'string', 'default' => 'ASC'],
						'orderby' => ['type' => 'string', 'default' => 'date'],
						'showTitle' => ['type' => 'boolean', 'default' => true],
						'showExcerpt' => ['type' => 'boolean', 'default' => true],
						'showDate' => ['type' => 'boolean', 'default' => true],
						'showAuthor' => ['type' => 'boolean', 'default' => false],
						'showIcal' => ['type' => 'boolean', 'default' => true],
						'showImage' => ['type' => 'boolean', 'default' => true],
						'imageSize' => ['type' => 'string', 'default' => ''],
						'showButton' => ['type' => 'string', 'default' => 'yes'],
						'showButtonText' => ['type' => 'string', 'default' => __('Read more', 'cosmic-blocks')],
						'showButtonStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-primary'],
						'showButtonTarget' => ['type' => 'string', 'default' => '_self'],
						'showButtonIcal' => ['type' => 'string', 'default' => 'yes'],
						'showButtonIcalText' => ['type' => 'string', 'default' => __('Copy to calendar', 'cosmic-blocks')],
						'showButtonIcalStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-secondary'],
						'showPagination' => ['type' => 'boolean', 'default' => true],
						'paginationStyle' => ['type' => 'string', 'default' => 'cwp-pagination cwp-pagination-secondary'],
						'textcolor' => ['type' => 'string', 'default' => ''],
						'color' => ['type' => 'string', 'default' => ''],
					]
				]);
			}
		}

		// Events Upcoming List
		if(get_option('cosmic_blocks_events_upcoming_list') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/events-upcoming-list', [
					'editor_script' => 'cwp-events-upcoming-list-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
					'render_callback' => 'cwp_render_events_upcoming_list',
					'attributes' => [
						'preview' => ['type' => 'boolean', 'default' => false],
						'blockID' => ['type' => 'string', 'default' => ''],
						'columns' => ['type' => 'number', 'default' => 1],
						'container' => ['type' => 'string', 'default' => 'cwp-full-width-wide-container'],
						'startClass' => ['type' => 'string', 'default' => 'cwp-all-left'],
						'backgroundType' => ['type' => 'string', 'default' => 'color'],
						'backgroundTextColor' => ['type' => 'string', 'default' => ''],
						'solidColor' => ['type' => 'string', 'default' => ''],
						'gradient' => ['type' => 'string', 'default' => ''],
						'backgroundMediaID' => ['type' => 'number', 'default' => ''],
						'backgroundMediaURL' => ['type' => 'string', 'default' => ''],
						'animationStyle' => ['type' => 'string', 'default' => ''],
						'animationDelay' => ['type' => 'number', 'default' => 10],
						'threshold' => ['type' => 'number', 'default' => 40],
						'paddingBeforeIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterItems' => ['type' => 'number', 'default' => 50],
						'introsection' => ['type' => 'string', 'default' => 'empty'],
						'introtitle' => ['type' => 'string', 'default' => ''],
						'introsubtitle' => ['type' => 'string', 'default' => ''],
						'introduction' => ['type' => 'string', 'default' => ''],
						'boxradius' => ['type' => 'number', 'default' => 0],
						'boxradiusfix' => ['type' => 'boolean', 'default' => false],
						'boxpadding' => ['type' => 'number', 'default' => 0],
						'boxpaddingfix' => ['type' => 'boolean', 'default' => false],
						'postsPerPage' => ['type' => 'string', 'default' => '25'],
						'position' => ['type' => 'string', 'default' => 'inside'],
						'order' => ['type' => 'string', 'default' => 'ASC'],
						'orderby' => ['type' => 'string', 'default' => 'date'],
						'showTitle' => ['type' => 'boolean', 'default' => true],
						'showExcerpt' => ['type' => 'boolean', 'default' => true],
						'showDate' => ['type' => 'boolean', 'default' => true],
						'showAuthor' => ['type' => 'boolean', 'default' => false],
						'showIcal' => ['type' => 'boolean', 'default' => true],
						'showImage' => ['type' => 'boolean', 'default' => true],
						'imageSize' => ['type' => 'string', 'default' => ''],
						'showButton' => ['type' => 'string', 'default' => 'yes'],
						'showButtonText' => ['type' => 'string', 'default' => __('Read more', 'cosmic-blocks')],
						'showButtonStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-primary'],
						'showButtonTarget' => ['type' => 'string', 'default' => '_self'],
						'showButtonIcal' => ['type' => 'string', 'default' => 'yes'],
						'showButtonIcalText' => ['type' => 'string', 'default' => __('Copy to calendar', 'cosmic-blocks')],
						'showButtonIcalStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-secondary'],
						'showPagination' => ['type' => 'boolean', 'default' => true],
						'paginationStyle' => ['type' => 'string', 'default' => 'cwp-pagination cwp-pagination-secondary'],
						'textcolor' => ['type' => 'string', 'default' => ''],
						'color' => ['type' => 'string', 'default' => ''],
					]
				]);
			}
		}

		// Events Upcoming Tiles
		if(get_option('cosmic_blocks_events_upcoming_tiles') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/events-upcoming-tiles', [
					'editor_script' => 'cwp-events-upcoming-tiles-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
					'render_callback' => 'cwp_render_events_upcoming_tiles',
					'attributes' => [
						'preview' => ['type' => 'boolean', 'default' => false],
						'blockID' => ['type' => 'string', 'default' => ''],
						'columns' => ['type' => 'number', 'default' => 3],
						'container' => ['type' => 'string', 'default' => 'cwp-full-width-wide-container'],
						'backgroundType' => ['type' => 'string', 'default' => 'color'],
						'backgroundTextColor' => ['type' => 'string', 'default' => ''],
						'solidColor' => ['type' => 'string', 'default' => ''],
						'gradient' => ['type' => 'string', 'default' => ''],
						'backgroundMediaID' => ['type' => 'number', 'default' => ''],
						'backgroundMediaURL' => ['type' => 'string', 'default' => ''],
						'animationStyle' => ['type' => 'string', 'default' => ''],
						'animationDelay' => ['type' => 'number', 'default' => 10],
						'threshold' => ['type' => 'number', 'default' => 40],
						'paddingBeforeIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterItems' => ['type' => 'number', 'default' => 50],
						'introsection' => ['type' => 'string', 'default' => 'empty'],
						'introtitle' => ['type' => 'string', 'default' => ''],
						'introsubtitle' => ['type' => 'string', 'default' => ''],
						'introduction' => ['type' => 'string', 'default' => ''],
						'boxradius' => ['type' => 'number', 'default' => 0],
						'boxradiusfix' => ['type' => 'boolean', 'default' => false],
						'boxpadding' => ['type' => 'number', 'default' => 0],
						'boxpaddingfix' => ['type' => 'boolean', 'default' => false],
						'postsPerPage' => ['type' => 'string', 'default' => '24'],
						'position' => ['type' => 'string', 'default' => 'after'],
						'order' => ['type' => 'string', 'default' => 'ASC'],
						'orderby' => ['type' => 'string', 'default' => 'date'],
						'showTitle' => ['type' => 'boolean', 'default' => true],
						'showExcerpt' => ['type' => 'boolean', 'default' => true],
						'showDate' => ['type' => 'boolean', 'default' => true],
						'showAuthor' => ['type' => 'boolean', 'default' => false],
						'showIcal' => ['type' => 'boolean', 'default' => true],
						'showImage' => ['type' => 'boolean', 'default' => true],
						'imageSize' => ['type' => 'string', 'default' => ''],
						'showButton' => ['type' => 'string', 'default' => 'yes'],
						'showButtonText' => ['type' => 'string', 'default' => __('Read more', 'cosmic-blocks')],
						'showButtonStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-primary'],
						'showButtonTarget' => ['type' => 'string', 'default' => '_self'],
						'showButtonIcal' => ['type' => 'string', 'default' => 'yes'],
						'showButtonIcalText' => ['type' => 'string', 'default' => __('Copy to calendar', 'cosmic-blocks')],
						'showButtonIcalStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-secondary'],
						'showPagination' => ['type' => 'boolean', 'default' => true],
						'paginationStyle' => ['type' => 'string', 'default' => 'cwp-pagination cwp-pagination-secondary'],
						'textcolor' => ['type' => 'string', 'default' => ''],
						'color' => ['type' => 'string', 'default' => ''],
					]
				]);
			}
		}

		// Facts
		if(get_option('cosmic_blocks_facts') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/facts', [
					'editor_script' => 'cwp-facts-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Foreword
		if(get_option('cosmic_blocks_foreword') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/foreword', [
					'editor_script' => 'cwp-foreword-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Greeting
		if(get_option('cosmic_blocks_greeting') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/greeting', [
					'editor_script' => 'cwp-greeting-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Image
		if(get_option('cosmic_blocks_image') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/image', [
					'editor_script' => 'cwp-image-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Inline Notice
		if(get_option('cosmic_blocks_inline_notice') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/inline-notice', [
					'editor_script' => 'cwp-inline-notice-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Launch
		if(get_option('cosmic_blocks_launch') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/launch', [
					'editor_script' => 'cwp-launch-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Like/Dislike
		if(get_option('cosmic_blocks_like_dislike') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/like-dislike', [
					'editor_script' => 'cwp-like-dislike-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Links Download
		if(get_option('cosmic_blocks_linklist_downloads') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/links-download', [
					'editor_script' => 'cwp-links-download-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Links Icon
		if(get_option('cosmic_blocks_linklist_icons') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/links-icon', [
					'editor_script' => 'cwp-links-icon-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Links Image
		if(get_option('cosmic_blocks_linklist_images') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/blocks-links-image', [
					'editor_script' => 'cwp-links-image-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Links Image Slide
		if(get_option('cosmic_blocks_linklist_images_slide') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/blocks-links-image-slide', [
					'editor_script' => 'cwp-links-image-slide-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Links List
		if(get_option('cosmic_blocks_links_list') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/blocks-links-list', [
					'editor_script' => 'cwp-links-list-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// OpenStreetMap
		/* if(get_option('cosmic_blocks_openstreetmap') === '1')
		  {
		  if(function_exists('register_block_type'))
		  {
		  register_block_type('cwp/blocks-openstreetmap', [
		  'editor_script' => 'cwp-openstreetmap-script',
		  'editor_style' => 'cwp-editor-style',
		  'style' => 'cwp-base-style',
		  ]);
		  }
		  } */

		// Posts List
		if(get_option('cosmic_blocks_posts_list') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/posts-list', [
					'editor_script' => 'cwp-posts-list-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
					'render_callback' => 'cwp_render_posts_list',
					'attributes' => [
						'preview' => ['type' => 'boolean', 'default' => false],
						'blockID' => ['type' => 'string', 'default' => ''],
						'columns' => ['type' => 'number', 'default' => 1],
						'container' => ['type' => 'string', 'default' => 'cwp-full-width-wide-container'],
						'startClass' => ['type' => 'string', 'default' => 'cwp-all-left'],
						'backgroundType' => ['type' => 'string', 'default' => 'color'],
						'backgroundTextColor' => ['type' => 'string', 'default' => ''],
						'solidColor' => ['type' => 'string', 'default' => ''],
						'gradient' => ['type' => 'string', 'default' => ''],
						'backgroundMediaID' => ['type' => 'number', 'default' => ''],
						'backgroundMediaURL' => ['type' => 'string', 'default' => ''],
						'animationStyle' => ['type' => 'string', 'default' => ''],
						'animationDelay' => ['type' => 'number', 'default' => 10],
						'threshold' => ['type' => 'number', 'default' => 40],
						'paddingBeforeIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterItems' => ['type' => 'number', 'default' => 50],
						'introsection' => ['type' => 'string', 'default' => 'empty'],
						'introtitle' => ['type' => 'string', 'default' => ''],
						'introsubtitle' => ['type' => 'string', 'default' => ''],
						'introduction' => ['type' => 'string', 'default' => ''],
						'boxradius' => ['type' => 'number', 'default' => 0],
						'boxradiusfix' => ['type' => 'boolean', 'default' => false],
						'boxpadding' => ['type' => 'number', 'default' => 0],
						'boxpaddingfix' => ['type' => 'boolean', 'default' => false],
						'postsPerPage' => ['type' => 'string', 'default' => '25'],
						'position' => ['type' => 'string', 'default' => 'inside'],
						'order' => ['type' => 'string', 'default' => 'ASC'],
						'orderby' => ['type' => 'string', 'default' => 'date'],
						'showTitle' => ['type' => 'boolean', 'default' => true],
						'showExcerpt' => ['type' => 'boolean', 'default' => true],
						'excerptCharlength' => ['type' => 'number', 'default' => 120],
						'showDate' => ['type' => 'boolean', 'default' => true],
						'showAuthor' => ['type' => 'boolean', 'default' => true],
						'showImage' => ['type' => 'boolean', 'default' => true],
						'imageSize' => ['type' => 'string', 'default' => ''],
						'showButton' => ['type' => 'string', 'default' => 'yes'],
						'showButtonText' => ['type' => 'string', 'default' => __('Read more', 'cosmic-blocks')],
						'showButtonStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-secondary'],
						'showButtonTarget' => ['type' => 'string', 'default' => '_self'],
						'selectedCategory' => ['type' => 'string', 'default' => ''],
						'showPagination' => ['type' => 'boolean', 'default' => true],
						'paginationStyle' => ['type' => 'string', 'default' => 'cwp-pagination cwp-pagination-secondary'],
						'textcolor' => ['type' => 'string', 'default' => ''],
						'color' => ['type' => 'string', 'default' => ''],
					]
				]);
			}
		}

		// Posts Tiles
		if(get_option('cosmic_blocks_posts_tiles') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/posts-tiles', [
					'editor_script' => 'cwp-posts-tiles-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
					'render_callback' => 'cwp_render_posts_tiles',
					'attributes' => [
						'preview' => ['type' => 'boolean', 'default' => false],
						'blockID' => ['type' => 'string', 'default' => ''],
						'columns' => ['type' => 'string', 'default' => '3'],
						'container' => ['type' => 'string', 'default' => 'cwp-full-width-wide-container'],
						'backgroundType' => ['type' => 'string', 'default' => 'color'],
						'backgroundTextColor' => ['type' => 'string', 'default' => ''],
						'solidColor' => ['type' => 'string', 'default' => ''],
						'gradient' => ['type' => 'string', 'default' => ''],
						'backgroundMediaID' => ['type' => 'number', 'default' => ''],
						'backgroundMediaURL' => ['type' => 'string', 'default' => ''],
						'animationStyle' => ['type' => 'string', 'default' => ''],
						'animationDelay' => ['type' => 'number', 'default' => 10],
						'threshold' => ['type' => 'number', 'default' => 40],
						'paddingBeforeIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterItems' => ['type' => 'number', 'default' => 50],
						'introsection' => ['type' => 'string', 'default' => 'empty'],
						'introtitle' => ['type' => 'string', 'default' => ''],
						'introsubtitle' => ['type' => 'string', 'default' => ''],
						'introduction' => ['type' => 'string', 'default' => ''],
						'boxradius' => ['type' => 'number', 'default' => 0],
						'boxradiusfix' => ['type' => 'boolean', 'default' => false],
						'boxpadding' => ['type' => 'number', 'default' => 0],
						'boxpaddingfix' => ['type' => 'boolean', 'default' => false],
						'postsPerPage' => ['type' => 'string', 'default' => '24'],
						'position' => ['type' => 'string', 'default' => 'after'],
						'order' => ['type' => 'string', 'default' => 'ASC'],
						'orderby' => ['type' => 'string', 'default' => 'date'],
						'showTitle' => ['type' => 'boolean', 'default' => true],
						'showExcerpt' => ['type' => 'boolean', 'default' => true],
						'excerptCharlength' => ['type' => 'number', 'default' => 120],
						'showDate' => ['type' => 'boolean', 'default' => true],
						'showAuthor' => ['type' => 'boolean', 'default' => true],
						'showImage' => ['type' => 'boolean', 'default' => true],
						'imageSize' => ['type' => 'string', 'default' => ''],
						'showButton' => ['type' => 'string', 'default' => 'yes'],
						'showButtonText' => ['type' => 'string', 'default' => __('Read more', 'cosmic-blocks')],
						'showButtonStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-secondary'],
						'showButtonTarget' => ['type' => 'string', 'default' => '_self'],
						'selectedCategory' => ['type' => 'string', 'default' => ''],
						'showPagination' => ['type' => 'boolean', 'default' => true],
						'paginationStyle' => ['type' => 'string', 'default' => 'cwp-pagination cwp-pagination-secondary'],
						'textcolor' => ['type' => 'string', 'default' => ''],
						'color' => ['type' => 'string', 'default' => ''],
					]
				]);
			}
		}

		// Pricebox
		if(get_option('cosmic_blocks_pricebox') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/pricebox', [
					'editor_script' => 'cwp-pricebox-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Pricelist
		if(get_option('cosmic_blocks_pricelist') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/pricelist', [
					'editor_script' => 'cwp-pricelist-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Progress
		if(get_option('cosmic_blocks_progress') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/progress', [
					'editor_script' => 'cwp-progress-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Separator
		if(get_option('cosmic_blocks_separator') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/separator', [
					'editor_script' => 'cwp-separator-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Showcase
		if(get_option('cosmic_blocks_showcase') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/showcase', [
					'editor_script' => 'cwp-showcase-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Social Share
		if(get_option('cosmic_blocks_social_share') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/social-share', [
					'editor_script' => 'cwp-social-share-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
					'render_callback' => 'cwp_render_social_share',
					'attributes' => [
						'preview' => ['type' => 'boolean', 'default' => false],
						'blockID' => ['type' => 'string', 'default' => ''],
						'columns' => ['type' => 'number', 'default' => 1],
						'buttonStyle' => ['type' => 'string', 'default' => 'cwp-button cwp-button-white'],
						'buttonType' => ['type' => 'string', 'default' => 'text_icon'],
						'positionClass' => ['type' => 'string', 'default' => 'cwp-all-left'],
						'container' => ['type' => 'string', 'default' => 'cwp-full-width-wide-container'],
						'backgroundType' => ['type' => 'string', 'default' => 'color'],
						'backgroundTextColor' => ['type' => 'string', 'default' => ''],
						'solidColor' => ['type' => 'string', 'default' => ''],
						'gradient' => ['type' => 'string', 'default' => ''],
						'backgroundMediaID' => ['type' => 'number', 'default' => ''],
						'backgroundMediaURL' => ['type' => 'string', 'default' => ''],
						'animationStyle' => ['type' => 'string', 'default' => ''],
						'animationDelay' => ['type' => 'number', 'default' => 10],
						'threshold' => ['type' => 'number', 'default' => 40],
						'paddingBeforeIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterIntro' => ['type' => 'number', 'default' => 50],
						'paddingAfterItems' => ['type' => 'number', 'default' => 50],
						'introsection' => ['type' => 'string', 'default' => 'empty'],
						'introtitle' => ['type' => 'string', 'default' => ''],
						'introsubtitle' => ['type' => 'string', 'default' => ''],
						'introduction' => ['type' => 'string', 'default' => ''],
						'boxradius' => ['type' => 'number', 'default' => 0],
						'boxradiusfix' => ['type' => 'boolean', 'default' => false],
						'boxpadding' => ['type' => 'number', 'default' => 0],
						'boxpaddingfix' => ['type' => 'boolean', 'default' => false],
						'facebook' => ['type' => 'boolean', 'default' => true],
						'twitter' => ['type' => 'boolean', 'default' => true],
						'linkedin' => ['type' => 'boolean', 'default' => true],
						'whatsapp' => ['type' => 'boolean', 'default' => true],
						'xing' => ['type' => 'boolean', 'default' => true]
					]
				]);
			}
		}

		// Split List
		if(get_option('cosmic_blocks_split_list') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/split-list', [
					'editor_script' => 'cwp-split-list-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Statement
		if(get_option('cosmic_blocks_statement') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/statement', [
					'editor_script' => 'cwp-statement-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Steps
		if(get_option('cosmic_blocks_steps') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/steps', [
					'editor_script' => 'cwp-steps-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Testimonial
		if(get_option('cosmic_blocks_testimonial') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/testimonial', [
					'editor_script' => 'cwp-testimonial-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Title & Intro
		if(get_option('cosmic_blocks_title_intro') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/title-intro', [
					'editor_script' => 'cwp-title-intro-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Voting Slogan
		if(get_option('cosmic_blocks_voting_slogans') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/voting-slogan', [
					'editor_script' => 'cwp-voting-slogan-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}

		// Wanted
		if(get_option('cosmic_blocks_wanted') === '1')
		{
			if(function_exists('register_block_type'))
			{
				register_block_type('cwp/wanted', [
					'editor_script' => 'cwp-wanted-script',
					'editor_style' => 'cwp-editor-style',
					'style' => 'cwp-base-style',
				]);
			}
		}
	}

	/**
	 * Show update notice
	 */
	function update_notice($upgrader_object, $options)
	{
		$our_plugin = plugin_basename(__FILE__);

		// If an update has taken place and the updated type is plugins and the plugins element exists
		if($options['action'] === 'update' && $options['type'] === 'plugin' && isset($options['plugins']))
		{
			foreach($options['plugins'] as $plugin)
			{
				if($plugin == $our_plugin)
				{
					// Set a transient to record that our plugin has just been updated
					set_transient('wp_cosmic_blocks_updated', 1);
				}
			}
		}

		// fire activate function again to register new settings
		CosmicBlocks_Register::activate();
	}

	function update_admin_notices()
	{
		// Check the transient to see if we've just updated the plugin
		if(get_transient('wp_cosmic_blocks_updated'))
		{
			echo '<div class="notice notice-success">' . __('Thanks for updating cosmic blocks.', 'cosmic-blocks') . '</div>';
			delete_transient('wp_cosmic_blocks_updated');
		}
	}

	/**
	 * Add dependency check message
	 */
	function admin_notice()
	{
		$checkPlugins = [
			'cosmic-campaigns' => [
				'name' => 'Cosmic Campaigns',
				'path' => 'cosmic-campaigns/cosmic-campaigns.php'
			]
		];

		$installedPlugins = get_plugins();

		$sitewidePlugins = [];
		if(is_array(get_site_option('active_sitewide_plugins')))
		{
			$sitewidePlugins = get_site_option('active_sitewide_plugins');
		}
		$plugins = array_merge(array_flip(get_option('active_plugins')), $sitewidePlugins);

		$message = '';
		foreach($checkPlugins as $checkPluginName => $checkPluginInfo)
		{
			if(!array_key_exists($checkPluginInfo['path'], $plugins))
			{
				if(isset($installedPlugins[$checkPluginInfo['path']]))
				{
					$message .= '<p>' . sprintf(__('The Plugin <strong>%s</strong> is not active but highly recommended.', 'lausanne-by-cosmic-wp'), $checkPluginInfo['name']) . ' <a class="button" href="' . self_admin_url('plugins.php') . '">' . __('Activate', 'lausanne-by-cosmic-wp') . '</a></p>';
				}
				else
				{
					$url = add_query_arg(['action' => 'install-plugin', 'plugin' => $checkPluginName], self_admin_url('update.php'));
					$message .= '<p>' . sprintf(__('The Plugin <strong>%s</strong> is missing but highly recommended.', 'cosmic-campaigns'), $checkPluginInfo['name']) . ' <a class="button" href="' . wp_nonce_url($url, 'install-plugin_' . $checkPluginName) . '">' . __('Install', 'lausanne-by-cosmic-wp') . '</a></p>';
				}
			}
		}

		if(!empty($message))
		{
			echo '<div id="cosmic_blocks_admin_notice" class="notice notice-error">';
			echo '<p><strong>' . __('Cosmic Blocks', 'cosmic-blocks') . '</strong></p>';
			echo '<p>' . __('Cosmic Blocks needs the following plugins for the best user experience.', 'cosmic-blocks') . '</p>';
			echo '<div class="cosmic_admin_notice_message">' . $message . '</div>';
			echo '<p><a href="admin.php?page=cosmic_wp_blocks" class="cosmic_blocks_admin_notice_dismiss">' . __('Disable notice', 'cosmic-blocks') . '</a></p>';
			echo '</div>';
		}
	}

	/**
	 * set dependency check disabled
	 */
	function wp_ajax_cosmic_blocks_admin_notice_dismiss()
	{
		update_option('cosmic_blocks_dependency_check', 'disabled');
		wp_die();
	}

	/**
	 * set dependency check disabled
	 */
	function wp_ajax_cosmic_blocks_thumbnail_helper()
	{
		$thumbs = [];
		if(isset($_POST['items']) && isset($_POST['thumb']))
		{
			foreach($_POST['items'] as $index => $item)
			{
				if(!empty($item['mediaID']))
				{
					$full = wp_get_attachment_image_src(sanitize_key($item['mediaID']), 'full');
					$thumb = wp_get_attachment_image_src(sanitize_key($item['mediaID']), $_POST['thumb']);

					$thumbs[] = [
						'id' => $item['mediaID'],
						'index' => $index,
						'full' => [
							'src' => $full[0],
							'width' => $full[1],
							'height' => $full[2],
							'is_intermediate' => $full[3],
						],
						'thumb' => [
							'src' => $thumb[0],
							'width' => $thumb[1],
							'height' => $thumb[2],
							'is_intermediate' => $thumb[3],
						]
					];
				}
			}
		}

		echo json_encode($thumbs);
		wp_die();
	}

	/**
	 * set dependency check disabled
	 */
	function wp_ajax_cosmic_blocks_thumbnail_helper_single()
	{
		$single = [];
		if(isset($_POST['mediaID']) && isset($_POST['thumb']))
		{
			$full = wp_get_attachment_image_src(sanitize_key($_POST['mediaID']), 'full');
			$thumb = wp_get_attachment_image_src(sanitize_key($_POST['mediaID']), $_POST['thumb']);

			$single = [
				'id' => $_POST['mediaID'],
				'full' => [
					'src' => $full[0],
					'width' => $full[1],
					'height' => $full[2],
					'is_intermediate' => $full[3],
				],
				'thumb' => [
					'src' => $thumb[0],
					'width' => $thumb[1],
					'height' => $thumb[2],
					'is_intermediate' => $thumb[3],
				]
			];
		}

		echo json_encode($single);
		wp_die();
	}

	function theme_setup()
	{
		add_theme_support('align-wide');
		add_theme_support('post-thumbnails');

		add_image_size('cosmic-thumbnail-50xAUTO', 50, false, false);
		add_image_size('cosmic-thumbnail-AUTOx50', false, 50, false);
		add_image_size('cosmic-thumbnail-100xAUTO', 100, false, false);
		add_image_size('cosmic-thumbnail-AUTOx100', false, 100, false);
		add_image_size('cosmic-thumbnail-100x100', 100, 100, true);
		add_image_size('cosmic-thumbnail-150x150', 150, 150, true);
		add_image_size('cosmic-thumbnail-300x225', 300, 225, true);
		add_image_size('cosmic-thumbnail-640x640', 640, 640, true);
		add_image_size('cosmic-thumbnail-AUTOx640', false, 640, false);
		add_image_size('cosmic-thumbnail-640xAUTO', 640, false, false);
		add_image_size('cosmic-thumbnail-960x960', 960, 960, true);
		add_image_size('cosmic-thumbnail-AUTOx960', false, 960, true);
		add_image_size('cosmic-thumbnail-960xAUTO', 960, false, true);
		add_image_size('cosmic-thumbnail-1024x768', 1024, 768, true);
		add_image_size('cosmic-thumbnail-AUTOx768', false, 768, false);
		add_image_size('cosmic-thumbnail-1024xAUTO', 1024, false, false);
		add_image_size('cosmic-thumbnail-1920x1080', 1920, 1080, true);
		add_image_size('cosmic-thumbnail-AUTOx1080', false, 1080, false);
		add_image_size('cosmic-thumbnail-1920xAUTO', 1920, false, false);
	}

	function custom_image_sizes($size_names)
	{
		$new_sizes = [
			'cosmic-thumbnail-100xAUTO' => __('100xAUTO', 'cosmic-blocks'),
			'cosmic-thumbnail-AUTOx100' => __('AUTOx100', 'cosmic-blocks'),
		];

		return array_merge($size_names, $new_sizes);
	}

	function thumbnail_upscale($default, $orig_w, $orig_h, $new_w, $new_h, $crop)
	{
		if(!$crop)
		{
			return null; // let the wordpress default function handle this
		}

		$aspect_ratio = $orig_w / $orig_h;
		$size_ratio = max($new_w / $orig_w, $new_h / $orig_h);

		$crop_w = round($new_w / $size_ratio);
		$crop_h = round($new_h / $size_ratio);

		$s_x = floor(($orig_w - $crop_w) / 2);
		$s_y = floor(($orig_h - $crop_h) / 2);

		return [0, 0, (int) $s_x, (int) $s_y, (int) $new_w, (int) $new_h, (int) $crop_w, (int) $crop_h];
	}

	static function the_excerpt_max_charlength($excerpt, $charlength)
	{
		$html = '';
		$charlength++;
		if(mb_strlen($excerpt) > $charlength)
		{
			$subex = mb_substr($excerpt, 0, $charlength - 5);
			$exwords = explode(' ', $subex);
			$excut = - (mb_strlen($exwords[count($exwords) - 1]));
			if($excut < 0)
			{
				$html .= mb_substr($subex, 0, $excut);
			}
			else
			{
				$html .= $subex;
			}
			$html .= '[...]';
		}
		else
		{
			$html .= $excerpt;
		}
		
		return $html;
	}
}

/**
 * 
 * Dynamic blocks render
 * 
 */
function cwp_render_contacts_list($attributes, $content)
{
	$shortcodeAttributes = implode(' ', array_map(function($k, $v) {
			return strtolower($k) . '="' . htmlspecialchars($v) . '"';
		}, array_keys($attributes), $attributes
	));

	$html = do_shortcode('[cwp_contacts_list ' . $shortcodeAttributes . ']');

	if($attributes['preview'] === true)
	{
		return $html;
	}
	return cwp_dynamicRender($attributes, 'wp-block-cwp-contacts-list', $html);
}

function cwp_render_contacts_tiles($attributes, $content)
{
	$shortcodeAttributes = implode(' ', array_map(function($k, $v) {
			return strtolower($k) . '="' . htmlspecialchars($v) . '"';
		}, array_keys($attributes), $attributes
	));

	$html = do_shortcode('[cwp_contacts_tiles ' . $shortcodeAttributes . ']');

	if($attributes['preview'] === true)
	{
		return $html;
	}
	return cwp_dynamicRender($attributes, 'wp-block-cwp-contacts-tiles', $html);
}

function cwp_render_contacts_target_bar($attributes, $content)
{
	$shortcodeAttributes = implode(' ', array_map(function($k, $v) {
			return strtolower($k) . '="' . htmlspecialchars($v) . '"';
		}, array_keys($attributes), $attributes
	));

	$html = do_shortcode('[cwp_contacts_target_bar ' . $shortcodeAttributes . ']');

	if($attributes['preview'] === true)
	{
		return $html;
	}
	return cwp_dynamicRender($attributes, 'wp-block-cwp-contacts-target-bar', $html);
}

function cwp_render_events_list($attributes, $content)
{
	$shortcodeAttributes = implode(' ', array_map(function($k, $v) {
			return strtolower($k) . '="' . htmlspecialchars($v) . '"';
		}, array_keys($attributes), $attributes
	));

	$html = do_shortcode('[cwp_events_list ' . $shortcodeAttributes . ']');

	if($attributes['preview'] === true)
	{
		return $html;
	}
	return cwp_dynamicRender($attributes, 'wp-block-cwp-events-list', $html);
}

function cwp_render_events_tiles($attributes, $content)
{
	$shortcodeAttributes = implode(' ', array_map(function($k, $v) {
			return strtolower($k) . '="' . htmlspecialchars($v) . '"';
		}, array_keys($attributes), $attributes
	));

	$html = do_shortcode('[cwp_events_tiles ' . $shortcodeAttributes . ']');

	if($attributes['preview'] === true)
	{
		return $html;
	}
	return cwp_dynamicRender($attributes, 'wp-block-cwp-events-tiles', $html);
}

function cwp_render_events_upcoming_list($attributes, $content)
{
	$shortcodeAttributes = implode(' ', array_map(function($k, $v) {
			return strtolower($k) . '="' . htmlspecialchars($v) . '"';
		}, array_keys($attributes), $attributes
	));

	$html = do_shortcode('[cwp_events_upcoming_list ' . $shortcodeAttributes . ']');

	if($attributes['preview'] === true)
	{
		return $html;
	}
	return cwp_dynamicRender($attributes, 'wp-block-cwp-events-upcoming-list', $html);
}

function cwp_render_events_upcoming_tiles($attributes, $content)
{
	$shortcodeAttributes = implode(' ', array_map(function($k, $v) {
			return strtolower($k) . '="' . htmlspecialchars($v) . '"';
		}, array_keys($attributes), $attributes
	));

	$html = do_shortcode('[cwp_events_upcoming_tiles ' . $shortcodeAttributes . ']');

	if($attributes['preview'] === true)
	{
		return $html;
	}
	return cwp_dynamicRender($attributes, 'wp-block-cwp-events-upcoming-tiles', $html);
}

function cwp_render_posts_list($attributes, $content)
{
	$shortcodeAttributes = implode(' ', array_map(function($k, $v) {
			return strtolower($k) . '="' . htmlspecialchars($v) . '"';
		}, array_keys($attributes), $attributes
	));

	$html = do_shortcode('[cwp_posts_list ' . $shortcodeAttributes . ']');

	if($attributes['preview'] === true)
	{
		return $html;
	}
	return cwp_dynamicRender($attributes, 'wp-block-cwp-posts-list', $html);
}

function cwp_render_posts_tiles($attributes, $content)
{
	$shortcodeAttributes = implode(' ', array_map(function($k, $v) {
			return strtolower($k) . '="' . htmlspecialchars($v) . '"';
		}, array_keys($attributes), $attributes
	));

	$html = do_shortcode('[cwp_posts_tiles ' . $shortcodeAttributes . ']');

	if($attributes['preview'] === true)
	{
		return $html;
	}
	return cwp_dynamicRender($attributes, 'wp-block-cwp-posts-tiles', $html);
}

function cwp_render_social_share($attributes, $content)
{
	$shortcodeAttributes = implode(' ', array_map(function($k, $v) {
			return strtolower($k) . '="' . htmlspecialchars($v) . '"';
		}, array_keys($attributes), $attributes
	));

	$html = do_shortcode('[cwp_social_share ' . $shortcodeAttributes . ']');

	if($attributes['preview'] === true)
	{
		return $html;
	}
	return cwp_dynamicRender($attributes, 'wp-block-cwp-social-share', $html);
}

function cwp_dynamicRender($attributes, $blockname, $shortcode)
{
	$stylePaddingAfterIntro = '';
	if(!empty($attributes['paddingAfterIntro']) && $attributes['paddingAfterIntro'] >= 0)
	{
		$stylePaddingAfterIntro .= ' padding-bottom:' . $attributes['paddingAfterIntro'] . 'px;';
	}

	$styleBackgroundTextColor = '';
	if(!empty($attributes['backgroundTextColor']))
	{
		$styleBackgroundTextColor .= ' color:' . $attributes['backgroundTextColor'] . ';';
	}

	$styleBeforeAfter = '';
	if(!empty($attributes['paddingBeforeIntro']) && $attributes['paddingBeforeIntro'] >= 0)
	{
		$styleBeforeAfter .= ' padding-top:' . $attributes['paddingBeforeIntro'] . 'px;';
	}
	if(!empty($attributes['paddingAfterItems']) && $attributes['paddingAfterItems'] >= 0)
	{
		$styleBeforeAfter .= ' padding-bottom:' . $attributes['paddingAfterItems'] . 'px;';
	}

	$styleBackground = '';
	if(!empty($attributes['backgroundType']) && $attributes['backgroundType'] === 'color')
	{
		$styleBackground .= ' background:' . $attributes['solidColor'] . ';';
	}
	else if(!empty($attributes['backgroundType']) && $attributes['backgroundType'] === 'gradient')
	{
		$styleBackground .= ' background:linear-gradient(' . $attributes['gradient'] . ');';
	}
	else if(!empty($attributes['backgroundType']) && $attributes['backgroundType'] === 'image')
	{
		$styleBackground .= ' background:url(' . $attributes['backgroundMediaURL'] . '); background-size:cover; background-position:center;';
	}

	if($attributes['introsection'] === 'empty')
	{
		if(!empty($attributes['introtitle']) || !empty($attributes['introsubtitle']) || !empty($attributes['introduction']))
		{
			$attributes['introsection'] = 'enabled';
		}
		else
		{
			$attributes['introsection'] = 'disabled';
		}
	}

	$html = '';
	//$html .= '<pre>'.print_r($attributes, true).'</pre>';
	$html .= '<div id="' . $attributes['blockID'] . '" class="' . $blockname . ' cwp-common-main-wrap ' . $attributes['container'] . '" style="' . trim($styleBackground) . '">';
		$html .= '<div class="cwp-common-main-wrap-inner" style="' . trim($styleBeforeAfter) . '" data-scroll-class="' . $attributes['animationStyle'] . '" data-scroll-delay="' . $attributes['animationDelay'] . '" data-scroll-threshold="' . $attributes['threshold'] . '">';

			if($attributes['introsection'] === 'enabled')
			{
				$html .= '<div class="cwp-main-intro" style="' . trim($stylePaddingAfterIntro) . '">';
				if(!empty($attributes['introtitle']) || !empty($attributes['introsubtitle']))
				{
					$html .= '<div class="cwp-intro-title">';
					$html .= '<h3 style="' . trim($styleBackgroundTextColor) . '">';
					if(!empty($attributes['introtitle']))
					{
						$html .= $attributes['introtitle'];
					}
					if(!empty($attributes['introsubtitle']))
					{
						$html .= '<span>' . $attributes['introsubtitle'] . '</span>';
					}
					$html .= '</h3>';
					$html .= '</div>';
				}
				if(!empty($attributes['introduction']))
				{
					$html .= '<div class="cwp-intro-introduction">';
						$html .= '<div class="cwp-intro-input">';
							$html .= '<p style="' . trim($styleBackgroundTextColor) . '">' . $attributes['introduction'] . '</p>';
						$html .= '</div>';
					$html .= '</div>';
				}
				$html .= '</div>';
			}

			$html .= $shortcode;

		$html .= '</div>';
	$html .= '</div>';

	return $html;
}
